/*
 * Created on May 10, 2005
 * 
 * This class is part of a Java implementation of the parse tree validation
 * technique proposed by Buehrer, Weide and Sivilotti.  
 * It is designed to eliminate SQL Injection
 * attacks in web applications.
 * 
 * Copyright 2005 The Ohio State University
 * 
 * This implementation makes use of the publically available SQL parser for Java,
 * ZQL (http://www.experlog.com/gibello/zql/).
 * 
 * How To Use:
 *  (install ZQL)
 *  Connection Conn = SafeDriverManager.getConnection("jdbc:mysql://localhost/database1?user=user1&password=pass");
 * 	String query = SQLGuard.init() + "SELECT * FROM table where col1 = " +  SQLGuard.wrap(id) ;
 * 	Statement s2 = Conn.createStatement();
 * 	RS = s2.executeQuery(query);
 * 
 */
package edu.osu.cse.SQLprevention;

import java.io.ByteArrayInputStream;
import java.util.Vector;
import Zql.ZConstant;
import Zql.ZDelete;
import Zql.ZExp;
import Zql.ZExpression;
import Zql.ZFromItem;
import Zql.ZInsert;
import Zql.ZQuery;
import Zql.ZSelectItem;
import Zql.ZStatement;
import Zql.ZUpdate;
import Zql.ZqlParser;
import java.util.Random;
//import java.util.HashMap;
import java.util.concurrent.ConcurrentHashMap;

/**
 * SQLGuard provides an interface for injection-safety in SQL strings.
 * <BR>
 * By using a parse tree representation of the SQL string, the class
 * can compare the structure both before and after user input has
 * been supplied, and raise an error if the two trees do not match.
 * <BR>
 * SQLGuard is part of a Java implementation of the parse tree validation
 * technique proposed by Buehrer, Weide and Sivilotti.  
 * It is designed to eliminate SQL Injection
 * attacks in web applications.
 * <BR>
 * This implementation makes use of the publically available SQL parser for Java,
 * ZQL (http://www.experlog.com/gibello/zql/).
 * <BR><BR>
 * How To Use:<BR>
 *  (install ZQL)<BR>
 *  Connection Conn = SafeDriverManager.getConnection("jdbc:mysql://localhost/database1?user=user1&password=pass");<BR>
 * 	String query = SQLGuard.init() + "SELECT * FROM table where col1 = " +  SQLGuard.wrap(id) ;<BR>
 * 	Statement s2 = Conn.createStatement();<BR>
 * 	RS = s2.executeQuery(query);<BR>
 * <BR>
 * Copyright 2005 The Ohio State University<BR>
 * <BR>
 * @author Paolo Sivilotti.
 * 
 * @author Greg Buehrer.
 * 
 * @version 0.6, August 28, 2005.
 */
public class SQLGuard {

	/**
	 * a concurrent hash table to store thread-key pairs.
	 * The defaults are used, namely 16 positions and a .75 fill factor.
	 */	
	private static ConcurrentHashMap keys;
	/**
	 * the number of bits to use for keys
	 */	
	private static int keySize=64;
	
	/*
	private static String access_results;
	*/
	
	/**
	 * an SQL string of the last query executed through SQLGuard
	 */	
	private static String lastQuery="";
	/**
	 * used to generate random keys
	 */	
	private static Random rand = new Random(System.currentTimeMillis());
     
	/**
	 * The default constructor, it instantiates the keys hashmap.
	 */	
	SQLGuard() {
        //super();
        keys = new ConcurrentHashMap(); //default size is 16, fill factor .75
    }
        
    


    /**
     * Generates a new key for this query.  
     * This key will be used for wrapping user input in the sql query.
     * Each new call to init creates a new key for the calling thread.
     * 
     * @return The key represented as a string, which must be appended to 
     * 		front of the querystring.
     */
    public static synchronized String init() {
    	if(keys==null)
    	{
    		keys = new ConcurrentHashMap();
    	}
    	long Key = rand.nextLong();
    	int inc = keySize / 4;
    	String local_key="";
    	for(int i=0;i<inc;i++)
    	{
    		long x = Key << (i*4);
    		x = x >>> 60;
    		local_key+=Long.toHexString(x);
    	}
    	//add to the hash table
    	keys.put(String.valueOf(Thread.currentThread().getId()),local_key);
        return local_key;
    }

    /**
     * Wraps a string variable.
     * Adds the key to either side of the variable.
     * For example a3e41831bc2143e1grega3e41831bc2143e1, where
     * a3e41831bc2143e1 is the key, and greg is the variable.
     * @param 
     * 		s a string representing a user supplied portion of the sql query
     * @return 
     * 		a wrapped string 
     */
    public static synchronized String wrap(String s) {
    	Object temp = keys.get(String.valueOf(Thread.currentThread().getId()));
    	return temp + s + temp;
    }

    /**
     * Wraps a long variable.
     * Adds the key to either side of the variable.
     * For example a3e41831bc2143e1555a3e41831bc2143e1, where
     * a3e41831bc2143e1 is the key, and 555 is the variable.
     * @param 
     * 		n a long, which is a user-supplied portion of the sql query
     * @return 
     * 		a wrapped string representation of the input 
     */
    
    public static String wrap(long n) {
    	Object temp = keys.get(String.valueOf(Thread.currentThread().getId() ));
        return "" + temp  + n + temp;
    }
    
    /**
     * Wraps a double variable.
     * Adds the key to either side of the variable.
     * For example a3e41831bc2143e1555.25a3e41831bc2143e1, where
     * a3e41831bc2143e1 is the key, and 555.25 is the variable.
     * @param 
     * 		n a double, which is a user-supplied portion of the sql query
     * @return 
     * 		a wrapped string representation of the input 
     */
    
    public static String wrap(double n) {
    	Object temp = keys.get(String.valueOf(Thread.currentThread().getId() ));
        return "" + temp  + n + temp;
    }
    /**
     * Wraps a char variable.
     * Adds the key to either side of the variable.
     * For example a3e41831bc2143e1Xa3e41831bc2143e1, where
     * a3e41831bc2143e1 is the key, and X is the variable.
     * @param 
     * 		n a char, which is a user-supplied portion of the sql query
     * @return 
     * 		a wrapped string representation of the input 
     */
    
    public static String wrap(char n) {
    	Object temp = keys.get(String.valueOf(Thread.currentThread().getId() ));
        return "" + temp  + n + temp;
    }
    /**
     * Wraps a short variable.
     * Adds the key to either side of the variable.
     * For example a3e41831bc2143e1555a3e41831bc2143e1, where
     * a3e41831bc2143e1 is the key, and 555 is the variable.
     * @param 
     * 		n a short, which is a user-supplied portion of the sql query
     * @return 
     * 		a wrapped string representation of the input 
     */
    
    public static String wrap(short n) {
    	Object temp = keys.get(String.valueOf(Thread.currentThread().getId() ));
        return "" + temp  + n + temp;
    }
    /**
     * Wraps a boolean variable.
     * Adds the key to either side of the variable.
     * For example a3e41831bc2143e1truea3e41831bc2143e1, where
     * a3e41831bc2143e1 is the key, and true is the variable.
     * @param 
     * 		n a boolean, which is a user-supplied portion of the sql query
     * @return 
     * 		a wrapped string representation of the input 
     */
    
    public static String wrap(boolean n) {
    	Object temp = keys.get(String.valueOf(Thread.currentThread().getId() ));
        return "" + temp  + n + temp;
    }
    /**
     * Wraps a float variable.
     * Adds the key to either side of the variable.
     * For example a3e41831bc2143e1555.25a3e41831bc2143e1, where
     * a3e41831bc2143e1 is the key, and 555.25 is the variable.
     * @param 
     * 		n a float, which is a user-supplied portion of the sql query
     * @return 
     * 		a wrapped string representation of the input 
     */
    
    public static String wrap(float n) {
    	Object temp = keys.get(String.valueOf(Thread.currentThread().getId() ));
        return "" + temp  + n + temp;
    }
    /**
     * Wraps an int variable.
     * Adds the key to either side of the variable.
     * For example a3e41831bc2143e1555a3e41831bc2143e1, where
     * a3e41831bc2143e1 is the key, and 555 is the variable.
     * @param 
     * 		n an int, which is a user-supplied portion of the sql query
     * @return 
     * 		a wrapped string representation of the input 
     */
    
    public static String wrap(int n) {
    	Object temp = keys.get(String.valueOf(Thread.currentThread().getId() ));
        return "" + temp  + n + temp;
    }  
    /**
     * Removes the key from the sql string, and then removes all wrappings from the query.
     * 
     * @param 
     * 		sql is an sql query
     * @return 
     * 		a clean sql query
     */  
    static String prepareForQuery(String sql) {
    	String local_key = sql.substring(0,(keySize / 4));
		sql = sql.substring((keySize / 4),sql.length());
        sql = removeWrappers(sql,"$2",local_key);
        return sql;
    }
	

   
    /**
     * Unwraps all wrappings in the sql query.
     * 
     * @param 
     * 		sql is an sql query
     * 		replacement is a regular expression of what to put back in
     * 		  it is a subgroup of the pattern, namely the one we wrapped
     * @return 
     * 		a clean sql query
     */  
	public static String removeWrappers(String sql, String replacement, String local_key)
	{
		String thePattern = "(" + local_key + ")(.*?)(" + local_key + ")";
		sql = sql.replaceAll(thePattern,replacement);
		//add a semicolon if it isnt there
		if(sql.charAt(sql.length()-1)!=';')
		{
			sql = sql + ";";
		}
		return sql;
	}

    /**
     * Confirms well-formedness of an SQL string.
     * Compares the parse trees for the SQL string before and after the
     * user input has been inserted.  If the two trees do not match exactly,
     * it returns false.  
     * 
     * @param sql
     *            a string representing an SQL query, decorated with special
     *            markers to set off user input
     * @return true if and only if the parse trees match (ie the parse trees
     *         with and without the special markers)
     */
	static boolean check(String sql) {
		
		String local_key = sql.substring(0,(keySize / 4));
		sql = sql.substring((keySize / 4),sql.length());
    	String str1 = removeWrappers(sql,"?",local_key);
    		
    	//removeAll(sql,local_key);
    	//String str2 = removeAllAndFill(sql,local_key);
    	String str2 = removeWrappers(sql,"$2",local_key);
		boolean x=false;
		boolean y=false;
		boolean z=false;
		
		//access_results="";
		try
		{

			ZqlParser p1 = new ZqlParser();
			ZqlParser p2 = new ZqlParser();
			
			p1.initParser( new ByteArrayInputStream(str1.getBytes()));
			p2.initParser( new ByteArrayInputStream(str2.getBytes()));
			ZStatement st1 = p1.readStatement();
			ZStatement st2 = p2.readStatement();
			//compare the two constants
			String constant1 = "";
			String constant2 = "";
			if(str1.indexOf(";")>=0)
			{
				constant1 = str1.substring(str1.indexOf(";")+1,str1.length());
			}
			if(str2.indexOf(";")>=0)
			{
				constant2 = str2.substring(str2.indexOf(";")+1,str2.length());
			}
			if(!constant1.equals(constant2)){return false;}
			//**********COMPARE QUERIES*****************************
			if(st1 instanceof ZQuery) // An SQL query
			{
				ZQuery q1 = (ZQuery) st1;
				ZQuery q2 = (ZQuery) st2;
				Vector sel1 = q1.getSelect(); // SELECT part of the query
				Vector sel2 = q2.getSelect(); // SELECT part of the query
				Vector from1 = q1.getFrom();  // FROM part of the query
				Vector from2 = q2.getFrom();  // FROM part of the query
				ZExpression where1 = (ZExpression)q1.getWhere();  // WHERE part of the query
				ZExpression where2 = (ZExpression)q2.getWhere();  // WHERE part of the query

				if(where1==null && where2==null){x=true;}
				else{x = compareWhere(where1,where2);}
				if(!x)
				{
					return false;
					//System.out.println("where was not equal");
				}
				y = compareSelect(sel1,sel2);
				if(!y)
				{
					//System.out.println("select was not equal");
					return false;
				}
				z = compareFrom(from1,from2);
				if(!z)
				{
					//System.out.println("from was not equal");
					return false;
				}
			}

		//**********COMPARE UPDATES*****************************
			else if(st1 instanceof ZUpdate && st2 instanceof ZUpdate) // An SQL update
			{
				ZUpdate u1 = (ZUpdate) st1;
				ZUpdate u2 = (ZUpdate) st2;

				String table1 = u1.getTable();  // TABLE part of the query
				String table2 = u2.getTable();  // TABLE part of the query
				//access_results += "*** WROTE TO TABLE " + u2.getTable() + " (UPDATE)\n";
				x = table1.equals(table2);
				if(x)
				{
					//System.out.println("table was equal");
				}
				else
				{
					//System.out.println("table was not equal");
					return false;
				}


				ZExpression where2 = (ZExpression)u2.getWhere();  // WHERE part of the query
				ZExpression where1 = (ZExpression)u1.getWhere();  // WHERE part of the query
				y = compareWhere(where1,where2);
				if(y)
				{
					//System.out.println("where was equal");
				}
				else
				{
					System.out.println("where was not equal");
					//equivalent=false;
					return false;
				}



				z=true;
				if(u1.getColumnUpdateCount()!=u2.getColumnUpdateCount())
				{
					z=false;
				}
				else
				{
					//System.out.println("column count equals " + u2.getColumnUpdateCount());
					for(int i=1;i<=u1.getColumnUpdateCount();i++)
					{
						System.out.println("column " + i);
						String c1=u1.getColumnUpdateName(i);
						String c2=u2.getColumnUpdateName(i);
						if(!c1.equals(c2))
						{
							//System.out.println("names not equal");
							return false;
							//z=false;
						}
						//System.out.println("names equal");
						if(!compareZExpression(u1.getColumnUpdate(i),u2.getColumnUpdate(i)))
						{
							z=false;
						}
					}
				}
				if(z)
				{
					//System.out.println("column update was equal");
				}
				else
				{
					System.out.println("column update was not equal");
					return false;
				}
			}//end else if


		//**********COMPARE DELETES*****************************
			else if(st1 instanceof ZDelete && st2 instanceof ZDelete) // An SQL update
			{
				ZDelete u1 = (ZDelete) st1;
				ZDelete u2 = (ZDelete) st2;

				String table1 = u1.getTable();  // TABLE part of the query
				String table2 = u2.getTable();  // TABLE part of the query
				//access_results += "*** WROTE TO TABLE " + u2.getTable() + " (DELETE)\n";

				x = table1.equals(table2);
				if(x)
				{
					//System.out.println("table was equal");
				}
				else
				{
					//System.out.println("table was not equal");
					return false;
				}


				ZExpression where2 = (ZExpression)u2.getWhere();  // WHERE part of the query
				ZExpression where1 = (ZExpression)u1.getWhere();  // WHERE part of the query
				y = compareWhere(where1,where2);
				if(y)
				{
					//System.out.println("where was equal");
				}
				else
				{
					//System.out.println("where was not equal");
					return false;
				}

				z=true;

			}//end else if


		//**********COMPARE INSERTS*****************************
			else if(st1 instanceof ZInsert && st2 instanceof ZInsert) // An SQL update
			{

				ZInsert u1 = (ZInsert) st1;
				ZInsert u2 = (ZInsert) st2;

				String table1 = u1.getTable();  // TABLE part
				String table2 = u2.getTable();  // TABLE part
				//access_results += "*** WROTE TO TABLE " + u2.getTable() + " (INSERT) \n";
				x = table1.equals(table2);
				if(x)
				{
					//System.out.println("table was equal");
				}
				else
				{
					//System.out.println("table was not equal");
					return false;
				}

				Vector cols1 = u1.getColumns(); // col list
				Vector cols2 = u2.getColumns(); // col list
				y=true;
				if(cols1.size()==cols2.size())
				{
					//System.out.println("size was the same ");
					for(int i=0;i<cols1.size();i++)
					{
						//System.out.println("COL " + i);
						if(!cols1.elementAt(i).equals(cols2.elementAt(i)))
						{
							//System.out.println("columns not the same ");
							return false;
							//y=false;
							//break;
						}
					}
				}
				else
				{
					y=false;
				}
				z=true;
				Vector vals1 = u1.getValues(); // SELECT part of the query
				Vector vals2 = u2.getValues(); // SELECT part of the query
				if(vals1.size()==vals2.size())
				{
					//System.out.println("size was the same " + vals1.size());
					for(int i=0;i<vals1.size();i++)
					{
						//System.out.println("COL " + i);
						if(!compareZExpression((ZExp)vals1.elementAt(i),(ZExp)vals2.elementAt(i)))
						{
							//System.out.println("values not the same ");
							return false;
							//z=false;
							//break;
						}
					}
				}
				else
				{
					z=false;
				}
			}//end else if
			return true;

		}
		catch(Exception e)
		{
	  		System.err.println("ERROR " + e.getClass());
	  		System.out.println("SQL not valid");
	  		return false;
		}
    }
    
	/**
	 * Compares two SELECT vectors, returning false if there 
	 * is not an exact match. 
	 *  
	 * The two must be the same size, and each pair must match.
	 * @param sel1 The SELECT vector with empty leaf nodes.
	 * @param sel2 The SELECT vector with user input supplied.
	 * @return true if they match, false if the vectors do not match.
	 */	
    private static boolean compareSelect(Vector sel1, Vector sel2)
	{
		if(sel1.size()!=sel2.size())
		{
			return false;
		}
		for(int i=0;i<sel1.size();i++)
		{
			if(!compareSelectItem((ZSelectItem)sel1.elementAt(i),(ZSelectItem)sel2.elementAt(i)))
			{
				return false;
			}
		}
		return true;
	}
	
	
    /**
     * Confirms whether two SELECT items are equal.
     * 		They must have the same number of subclauses.
     * 
     * @param sel1 
     * 			The first SELECT item.
     * @param sel2 
     * 			The second SELECT item, the one filled in by the user.
     * @return true if and only if the FROM items match.
     */
	private static boolean compareSelectItem(ZSelectItem sel1, ZSelectItem sel2)
	{
		if(sel1.isExpression() && sel2.isExpression())
		{
			return compareZExpression(sel1.getExpression(), sel2.getExpression());
		}
		else if(sel1.getAggregate()!=null || sel2.getAggregate()!=null)
		{
			return sel1.getAggregate().equals(sel2.getAggregate());
		}
		else if(sel1.isWildcard() && sel2.isWildcard())
		{
			return true;
		}
		else if(sel1.getColumn()!=null || sel2.getColumn()!=null)
		{
			return sel1.getColumn()==sel2.getColumn();
		}
		else return false;
	}


    /**
     * Confirms well-formedness of an SQL string's FROM clause.
     * 		They must have the same number of subclauses.
     * @param from1 
     * 			The first FROM clause (not filled in with user input).
     * @param from2 
     * 			The second FROM clause, filled in with user input.
     * @return true if and only if the FROM clause structure was maintained.
     */
	private static boolean compareFrom(Vector from1, Vector from2)
	{
		if(from1.size()!=from2.size())
		{
			return false;
		}
		for(int i=0;i<from1.size();i++)
		{
			if(!compareFromItem((ZFromItem)from1.elementAt(i),(ZFromItem)from2.elementAt(i)))
			{
				return false;
			}
		}
		return true;
	}


    /**
     * Confirms whether two FROM items are a match.
     * 		They must have the same number of subclauses,
     * 		and for each clause:<BR>
     * 				a) the column and table names match exactly<BR>
     * 				b) or both column names are missing and the table
     * 					names match.
     * 
     * @param from1 
     * 			The first FROM item.
     * @param from2 
     * 			The second FROM item, the one filled in by the user
     * @return true if and only if the FROM items match
     */
	private static boolean compareFromItem(ZFromItem from1, ZFromItem from2)
	{
		boolean result=true;
		if(from1.getColumn()!=null && from2.getColumn()!=null)
		{
			result= from1.getColumn().equals(from2.getColumn()) && from1.getTable().equals(from2.getTable());
		}
		else if (from1.getColumn()==null && from2.getColumn()==null)
		{
			result=from1.getTable().equals(from2.getTable());
		}
		else result=false;
		//access_results += "*** READ FROM TABLE " + from2.getTable() + "\n";
		return result;
	}


    /**
     * Confirms whether two EXPRESSIONS are a match.
     * It is the guts of the comparing, since most clauses are expressions.<BR>
     * It is recursive<BR>
     * 
     * 		
     * 		They must have the same number of subclauses, and for each clause<BR>
     * 				a) they must both be the same constant<BR>
     * 				b) or they must both be expressions, 
     * 					with matching operators and sub expressions<BR>
     * 				c) or the first must be a ? and the second must be a constant (a value).<BR>
     * 
     * @param exp1 
     * 			The first EXPRESSION item.
     * @param exp2 
     * 			The second EXPRESSION item, the one filled in by the user
     * @return true if and only if the EXPRESSION items match
     */
	private static boolean compareZExpression(ZExp exp1, ZExp exp2)
	{
		//System.out.println("in ZExp");
		boolean result=true;
		if(exp1 instanceof ZExpression && exp2 instanceof ZExpression)
		{

			Vector ops1 = ((ZExpression)exp1).getOperands();
			Vector ops2 = ((ZExpression)exp2).getOperands();
			if(ops1.size()==ops2.size())
			{
				if(ops1.size()==1)
				{
					//System.out.println("in ZExp 1");
					result = result && compareZExpression((ZExp)ops1.elementAt(0),(ZExp)ops2.elementAt(0));
				}
				else
				{
					//System.out.println("in ZExp 2");
					result = result && compareZExpression((ZExp)ops1.elementAt(0),(ZExp)ops2.elementAt(0));
					result = result && compareZExpression((ZExp)ops1.elementAt(1),(ZExp)ops2.elementAt(1));
					result = result && ((ZExpression)exp1).getOperator().equals(((ZExpression)exp2).getOperator());
				}
			}
			else
			{
				//System.out.println("not same size");
				result=false;
			}
		}
		else if(exp1 instanceof ZConstant && exp2 instanceof ZConstant)
		{
			//System.out.println("in ZCon");
			ZConstant c1 = (ZConstant) exp1;
			ZConstant c2 = (ZConstant) exp2;
			//System.out.println(c1.getValue() + " == " + c2.getValue());
			boolean holder = c1.getType()==c2.getType() && c1.getValue().equals(c2.getValue());
			//holder = holder || (c1.getValue().equals("QUERY_VARIABLE_HOLDER") && (c2.getType()==2 || c2.getType()==3));
			holder = holder || (c1.getValue().equals("?") && (c2.getType()==2 || c2.getType()==3));
			result=result && holder;

		}
		else if(exp1 instanceof ZExpression && exp2 instanceof ZConstant)
		{
			//System.out.println("in question mark");
			ZExpression e1 = (ZExpression) exp1;
			//ZConstant c2 = (ZConstant) exp2;
			//System.out.println(e1.toString() + " == " + c2.getValue() + " ?");
			result=result && e1.toString().equals("?");
		}
		else
		{
			//System.out.println("didnt match types");
			result=false;
		}
		return result;
	}
	

    /**
     * Confirms well-formedness of an SQL string's WHERE clause.
     * 
     * @param where1 
     * 			The first WHERE clause (not filled in with user input).
     * @param where2 
     * 			The second WHERE clause, filled in with user input.
     * @return true if and only if the WHERE clause structure was maintained.
     */
	private static boolean compareWhere(ZExp where1, ZExp where2)
	{
		if(compareZExpression(where1,where2))
		{
			return true;
		}
		else
		{
			return false;
		}
	}

  
	/**
	 * gets the size of the key (in bits).
	 * @return Returns the keySize in bits.
	 */
	public static int getKeySize() {
		return keySize;
	}
	/**
	 * Sets the keySize in bits, must be a multiple of 4.
	 * The default is 64 bits.
	 * @param keySize The keySize to set (in bits).
	 * 		
	 */
	public static void setKeySize(int keySize) {
		SQLGuard.keySize = keySize;
	}
	/**
	 * Gets the last query string which was executed through SQLGuard.
	 * It is set after a successful check call, and before statement.execute is called.
	 * Since it is global, multiple threads share it.
	 * @return Returns the lastQuery.
	 */
	public static String getLastQuery() {
		if(lastQuery==null)
			lastQuery="";
		return lastQuery;
	}
	/**
	 * @param lastQuery The lastQuery to set.
	 */
	public static synchronized void setLastQuery(String lastQuery) {
		SQLGuard.lastQuery = lastQuery;
	}
}

